@@ -239,7 +239,18 @@ static struct argp_option options[] = { { "debug", OPTION_DEBUG, 0, 0, "produce
 static void
 print_version (FILE *stream, struct argp_state *state arg_unused)
 {
-  cleanup_free char *rundir = libcrun_get_state_directory (arguments.root, NULL);
+  libcrun_error_t err = NULL;
+  cleanup_free char *rundir = NULL;
+  int ret;
+
+  ret = libcrun_get_state_directory (&rundir, arguments.root, NULL, &err);
+  if (UNLIKELY (ret < 0))
+    {
+      libcrun_error_release (&err);
+      fprintf (stderr, "Failed to get state directory\n");
+      exit (EXIT_FAILURE);
+    }
+
   fprintf (stream, "%s version %s\n", PACKAGE_NAME, PACKAGE_VERSION);
   fprintf (stream, "commit: %s\n", GIT_VERSION);
   fprintf (stream, "rundir: %s\n", rundir);
