@@ -121,37 +121,18 @@ xasprintf (char **str, const char *fmt, ...)
 int
 write_file_at_with_flags (int dirfd, int flags, mode_t mode, const char *name, const void *data, size_t len, libcrun_error_t *err)
 {
-  cleanup_close int fd = openat (dirfd, name, O_CLOEXEC | O_WRONLY | flags, mode);
+  cleanup_close int fd = -1;
   int ret = 0;
+
+  fd = openat (dirfd, name, O_CLOEXEC | O_WRONLY | flags, mode);
   if (UNLIKELY (fd < 0))
     return crun_make_error (err, errno, "opening file `%s` for writing", name);
 
-  if (len)
-    {
-      ret = TEMP_FAILURE_RETRY (write (fd, data, len));
-      if (UNLIKELY (ret < 0))
-        return crun_make_error (err, errno, "writing file `%s`", name);
-    }
+  ret = safe_write (fd, name, data, len, err);
+  if (UNLIKELY (ret < 0))
+    return ret;
 
-  return ret;
-}
-
-int
-write_file_at (int dirfd, const char *name, const void *data, size_t len, libcrun_error_t *err)
-{
-  return write_file_at_with_flags (dirfd, O_CLOEXEC | O_CREAT | O_TRUNC, 0700, name, data, len, err);
-}
-
-int
-write_file_with_flags (const char *name, int flags, const void *data, size_t len, libcrun_error_t *err)
-{
-  return write_file_at_with_flags (AT_FDCWD, flags, 0700, name, data, len, err);
-}
-
-int
-write_file (const char *name, const void *data, size_t len, libcrun_error_t *err)
-{
-  return write_file_with_flags (name, O_CREAT | O_TRUNC, data, len, err);
+  return (len > INT_MAX) ? INT_MAX : (int) len;
 }
 
 int
@@ -234,17 +215,17 @@ get_file_type (mode_t *mode, bool nofollow, const char *path)
 }
 
 int
-create_file_if_missing_at (int dirfd, const char *file, libcrun_error_t *err)
+create_file_if_missing_at (int dirfd, const char *file, mode_t mode, libcrun_error_t *err)
 {
-  cleanup_close int fd_write = openat (dirfd, file, O_CLOEXEC | O_CREAT | O_WRONLY, 0700);
+  cleanup_close int fd_write = openat (dirfd, file, O_CLOEXEC | O_CREAT | O_WRONLY, mode);
   if (fd_write < 0)
     {
-      mode_t mode;
+      mode_t tmp_mode;
       int ret;
 
       /* On errors, check if the file already exists.  */
-      ret = get_file_type_at (dirfd, &mode, false, file);
-      if (ret == 0 && S_ISREG (mode))
+      ret = get_file_type_at (dirfd, &tmp_mode, false, file);
+      if (ret == 0 && S_ISREG (tmp_mode))
         return 0;
 
       return crun_make_error (err, errno, "creating file `%s`", file);
@@ -349,12 +330,13 @@ close_and_replace (int *oldfd, int newfd)
 char *chroot_realpath (const char *chroot, const char *path, char resolved_path[]);
 
 static int
-safe_openat_fallback (int dirfd, const char *rootfs, size_t rootfs_len, const char *path,
-                      int flags, int mode, libcrun_error_t *err)
+safe_openat_fallback (int dirfd, const char *rootfs, const char *path, int flags,
+                      int mode, libcrun_error_t *err)
 {
   const char *path_in_chroot;
   cleanup_close int fd = -1;
   char buffer[PATH_MAX];
+  size_t rootfs_len = strlen (rootfs);
   int ret;
 
   path_in_chroot = chroot_realpath (rootfs, path, buffer);
@@ -389,7 +371,7 @@ safe_openat_fallback (int dirfd, const char *rootfs, size_t rootfs_len, const ch
 }
 
 int
-safe_openat (int dirfd, const char *rootfs, size_t rootfs_len, const char *path, int flags, int mode,
+safe_openat (int dirfd, const char *rootfs, const char *path, int flags, int mode,
              libcrun_error_t *err)
 {
   static bool openat2_supported = true;
@@ -406,7 +388,7 @@ safe_openat (int dirfd, const char *rootfs, size_t rootfs_len, const char *path,
           if (errno == ENOSYS)
             openat2_supported = false;
           if (errno == ENOSYS || errno == EINVAL || errno == EPERM)
-            return safe_openat_fallback (dirfd, rootfs, rootfs_len, path, flags, mode, err);
+            return safe_openat_fallback (dirfd, rootfs, path, flags, mode, err);
 
           return crun_make_error (err, errno, "openat2 `%s`", path);
         }
@@ -414,12 +396,13 @@ safe_openat (int dirfd, const char *rootfs, size_t rootfs_len, const char *path,
       return ret;
     }
 
-  return safe_openat_fallback (dirfd, rootfs, rootfs_len, path, flags, mode, err);
+  return safe_openat_fallback (dirfd, rootfs, path, flags, mode, err);
 }
 
 ssize_t
 safe_readlinkat (int dfd, const char *name, char **buffer, ssize_t hint, libcrun_error_t *err)
 {
+  /* Add 1 to make room for the NUL terminator.  */
   ssize_t buf_size = hint > 0 ? hint + 1 : 512;
   cleanup_free char *tmp_buf = NULL;
   ssize_t size;
@@ -429,8 +412,7 @@ safe_readlinkat (int dfd, const char *name, char **buffer, ssize_t hint, libcrun
       if (tmp_buf != NULL)
         buf_size += 256;
 
-      /* Allocate an extra byte so the buffer can be NUL terminated.  */
-      tmp_buf = xrealloc (tmp_buf, buf_size + 1);
+      tmp_buf = xrealloc (tmp_buf, buf_size);
 
       size = readlinkat (dfd, name, tmp_buf, buf_size);
       if (UNLIKELY (size < 0))
@@ -449,8 +431,7 @@ safe_readlinkat (int dfd, const char *name, char **buffer, ssize_t hint, libcrun
 
 static int
 crun_safe_ensure_at (bool do_open, bool dir, int dirfd, const char *dirpath,
-                     size_t dirpath_len, const char *path, int mode,
-                     int max_readlinks, libcrun_error_t *err)
+                     const char *path, int mode, int max_readlinks, libcrun_error_t *err)
 {
   cleanup_close int wd_cleanup = -1;
   cleanup_free char *npath = NULL;
@@ -516,7 +497,7 @@ crun_safe_ensure_at (bool do_open, bool dir, int dirfd, const char *dirpath,
                   if (LIKELY (ret >= 0))
                     {
                       return crun_safe_ensure_at (do_open, dir, dirfd,
-                                                  dirpath, dirpath_len,
+                                                  dirpath,
                                                   resolved_path, mode,
                                                   max_readlinks - 1, err);
                     }
@@ -542,7 +523,7 @@ crun_safe_ensure_at (bool do_open, bool dir, int dirfd, const char *dirpath,
             return crun_make_error (err, errno, "mkdir `/%s`", npath);
         }
 
-      cwd = safe_openat (dirfd, dirpath, dirpath_len, npath, (last_component ? O_PATH : 0) | O_CLOEXEC, 0, err);
+      cwd = safe_openat (dirfd, dirpath, npath, (last_component ? O_PATH : 0) | O_CLOEXEC, 0, err);
       if (UNLIKELY (cwd < 0))
         return crun_error_wrap (err, "creating `/%s`", path);
 
@@ -594,31 +575,30 @@ crun_safe_ensure_at (bool do_open, bool dir, int dirfd, const char *dirpath,
 }
 
 int
-crun_safe_create_and_open_ref_at (bool dir, int dirfd, const char *dirpath, size_t dirpath_len,
-                                  const char *path, int mode, libcrun_error_t *err)
+crun_safe_create_and_open_ref_at (bool dir, int dirfd, const char *dirpath, const char *path, int mode, libcrun_error_t *err)
 {
   int fd;
 
   /* If the file/dir already exists, just open it.  */
-  fd = safe_openat (dirfd, dirpath, dirpath_len, path, O_PATH | O_CLOEXEC, 0, err);
+  fd = safe_openat (dirfd, dirpath, path, O_PATH | O_CLOEXEC, 0, err);
   if (LIKELY (fd >= 0))
     return fd;
 
-  return crun_safe_ensure_at (true, dir, dirfd, dirpath, dirpath_len, path, mode, MAX_READLINKS, err);
+  return crun_safe_ensure_at (true, dir, dirfd, dirpath, path, mode, MAX_READLINKS, err);
 }
 
 int
-crun_safe_ensure_directory_at (int dirfd, const char *dirpath, size_t dirpath_len, const char *path, int mode,
+crun_safe_ensure_directory_at (int dirfd, const char *dirpath, const char *path, int mode,
                                libcrun_error_t *err)
 {
-  return crun_safe_ensure_at (false, true, dirfd, dirpath, dirpath_len, path, mode, MAX_READLINKS, err);
+  return crun_safe_ensure_at (false, true, dirfd, dirpath, path, mode, MAX_READLINKS, err);
 }
 
 int
-crun_safe_ensure_file_at (int dirfd, const char *dirpath, size_t dirpath_len, const char *path, int mode,
+crun_safe_ensure_file_at (int dirfd, const char *dirpath, const char *path, int mode,
                           libcrun_error_t *err)
 {
-  return crun_safe_ensure_at (false, false, dirfd, dirpath, dirpath_len, path, mode, MAX_READLINKS, err);
+  return crun_safe_ensure_at (false, false, dirfd, dirpath, path, mode, MAX_READLINKS, err);
 }
 
 int
@@ -627,33 +607,6 @@ crun_ensure_directory (const char *path, int mode, bool nofollow, libcrun_error_
   return crun_ensure_directory_at (AT_FDCWD, path, mode, nofollow, err);
 }
 
-int
-crun_ensure_file_at (int dirfd, const char *path, int mode, bool nofollow, libcrun_error_t *err)
-{
-  cleanup_free char *tmp = xstrdup (path);
-  size_t len = strlen (tmp);
-  char *it = tmp + len - 1;
-  int ret;
-
-  while (*it != '/' && it > tmp)
-    it--;
-  if (it > tmp)
-    {
-      *it = '\0';
-      ret = crun_ensure_directory_at (dirfd, tmp, mode, nofollow, err);
-      if (UNLIKELY (ret < 0))
-        return ret;
-      *it = '/';
-    }
-  return create_file_if_missing_at (dirfd, tmp, err);
-}
-
-int
-crun_ensure_file (const char *path, int mode, bool nofollow, libcrun_error_t *err)
-{
-  return crun_ensure_file_at (AT_FDCWD, path, mode, nofollow, err);
-}
-
 static int
 get_file_size (int fd, off_t *size)
 {
@@ -1056,6 +1009,31 @@ read_all_file (const char *path, char **out, size_t *len, libcrun_error_t *err)
   return read_all_file_at (AT_FDCWD, path, out, len, err);
 }
 
+int
+get_realpath_to_file (int dirfd, const char *path_name, char **absolute_path, libcrun_error_t *err)
+{
+  cleanup_close int targetfd = -1;
+
+  targetfd = TEMP_FAILURE_RETRY (openat (dirfd, path_name, O_RDONLY | O_CLOEXEC));
+  if (UNLIKELY (targetfd < 0))
+    return crun_make_error (err, errno, "error opening file `%s`", path_name);
+  else
+    {
+      ssize_t len;
+      proc_fd_path_t target_fd_path;
+
+      get_proc_self_fd_path (target_fd_path, targetfd);
+      len = safe_readlinkat (AT_FDCWD, target_fd_path, absolute_path, 0, err);
+      if (UNLIKELY (len < 0))
+        {
+          crun_error_release (err);
+          return crun_make_error (err, errno, "error unable to provide absolute path to file `%s`", path_name);
+        }
+    }
+
+  return 0;
+}
+
 int
 open_unix_domain_client_socket (const char *path, int dgram, libcrun_error_t *err)
 {
@@ -1448,6 +1426,7 @@ set_home_env (uid_t id)
   cleanup_free char *buf = NULL;
   long buf_size;
   cleanup_file FILE *stream = NULL;
+  int ret = -1;
 
   buf_size = sysconf (_SC_GETPW_R_SIZE_MAX);
   if (buf_size < 0)
@@ -1457,26 +1436,17 @@ set_home_env (uid_t id)
 
   stream = fopen ("/etc/passwd", "re");
   if (stream == NULL)
-    {
-      if (errno == ENOENT)
-        goto exit;
-
-      return -1;
-    }
+    goto error;
 
   for (;;)
     {
-      int ret;
       struct passwd *ret_pw = NULL;
 
       ret = fgetpwent_r (stream, &pwd, buf, buf_size, &ret_pw);
       if (UNLIKELY (ret != 0))
         {
-          if (errno == ENOENT)
-            return 0;
-
           if (errno != ERANGE)
-            return ret;
+            goto error;
 
           buf_size *= 2;
           buf = xrealloc (buf, buf_size);
@@ -1490,10 +1460,9 @@ set_home_env (uid_t id)
         }
     }
 
-exit:
-  /* If the user was not found, set it to something reasonable.  */
-  setenv ("HOME", "/", 1);
-  return 0;
+error:
+  /* Let callers handle the error if the user was not found. */
+  return ret;
 }
 
 /*if subuid or subgid exist, take the first range for the user */
@@ -2258,15 +2227,10 @@ find_annotation (libcrun_container_t *container, const char *name)
   return find_annotation_map (container->container_def->annotations, name);
 }
 
-ssize_t
-safe_write (int fd, const void *buf, ssize_t count)
+int
+safe_write (int fd, const char *fname, const void *buf, size_t count, libcrun_error_t *err)
 {
-  ssize_t written = 0;
-  if (count < 0)
-    {
-      errno = EINVAL;
-      return -1;
-    }
+  size_t written = 0;
   while (written < count)
     {
       ssize_t w = write (fd, buf + written, count - written);
@@ -2274,11 +2238,11 @@ safe_write (int fd, const void *buf, ssize_t count)
         {
           if (errno == EINTR || errno == EAGAIN)
             continue;
-          return w;
+          return crun_make_error (err, errno, "write file `%s`", fname);
         }
       written += w;
     }
-  return written;
+  return 0;
 }
 
 int
