@@ -27,6 +27,7 @@
 #include <dirent.h>
 #include <unistd.h>
 #include <signal.h>
+#include <fcntl.h>
 #include <ocispec/runtime_spec_schema_config_schema.h>
 #include <sys/wait.h>
 #include "container.h"
@@ -55,6 +56,8 @@
 #define LIKELY(x) __builtin_expect ((x), 1)
 #define UNLIKELY(x) __builtin_expect ((x), 0)
 
+#define WRITE_FILE_DEFAULT_FLAGS (O_CLOEXEC | O_CREAT | O_TRUNC | O_WRONLY)
+
 __attribute__ ((malloc)) static inline void *
 xmalloc (size_t size)
 {
@@ -257,28 +260,30 @@ int xasprintf (char **str, const char *fmt, ...) __attribute__ ((format (printf,
 
 int crun_path_exists (const char *path, libcrun_error_t *err);
 
-int write_file_with_flags (const char *name, int flags, const void *data, size_t len, libcrun_error_t *err);
-
-int write_file (const char *name, const void *data, size_t len, libcrun_error_t *err);
-
-int write_file_at (int dirfd, const char *name, const void *data, size_t len, libcrun_error_t *err);
-
 int write_file_at_with_flags (int dirfd, int flags, mode_t mode, const char *name, const void *data, size_t len, libcrun_error_t *err);
 
+static inline int
+write_file (const char *name, const void *data, size_t len, libcrun_error_t *err)
+{
+  return write_file_at_with_flags (AT_FDCWD, WRITE_FILE_DEFAULT_FLAGS, 0700, name, data, len, err);
+}
+
+static inline int
+write_file_at (int dirfd, const char *name, const void *data, size_t len, libcrun_error_t *err)
+{
+  return write_file_at_with_flags (dirfd, WRITE_FILE_DEFAULT_FLAGS, 0700, name, data, len, err);
+}
+
 int crun_ensure_directory (const char *path, int mode, bool nofollow, libcrun_error_t *err);
 
-int crun_ensure_file (const char *path, int mode, bool nofollow, libcrun_error_t *err);
-
 int crun_ensure_directory_at (int dirfd, const char *path, int mode, bool nofollow, libcrun_error_t *err);
 
-int crun_ensure_file_at (int dirfd, const char *path, int mode, bool nofollow, libcrun_error_t *err);
+int crun_safe_create_and_open_ref_at (bool dir, int dirfd, const char *dirpath, const char *path, int mode, libcrun_error_t *err);
 
-int crun_safe_create_and_open_ref_at (bool dir, int dirfd, const char *dirpath, size_t dirpath_len, const char *path, int mode, libcrun_error_t *err);
-
-int crun_safe_ensure_directory_at (int dirfd, const char *dirpath, size_t dirpath_len, const char *path, int mode,
+int crun_safe_ensure_directory_at (int dirfd, const char *dirpath, const char *path, int mode,
                                    libcrun_error_t *err);
 
-int crun_safe_ensure_file_at (int dirfd, const char *dirpath, size_t dirpath_len, const char *path, int mode,
+int crun_safe_ensure_file_at (int dirfd, const char *dirpath, const char *path, int mode,
                               libcrun_error_t *err);
 
 int crun_dir_p (const char *path, bool nofollow, libcrun_error_t *err);
@@ -287,7 +292,7 @@ int crun_dir_p_at (int dirfd, const char *path, bool nofollow, libcrun_error_t *
 
 int detach_process ();
 
-int create_file_if_missing_at (int dirfd, const char *file, libcrun_error_t *err);
+int create_file_if_missing_at (int dirfd, const char *file, mode_t mode, libcrun_error_t *err);
 
 int check_running_in_user_namespace (libcrun_error_t *err);
 
@@ -305,6 +310,8 @@ read_all_fd (int fd, const char *description, char **out, size_t *len, libcrun_e
   return read_all_fd_with_size_hint (fd, description, out, len, 0, err);
 }
 
+int get_realpath_to_file (int dirfd, const char *path_name, char **absolute_path, libcrun_error_t *err);
+
 int read_all_file (const char *path, char **out, size_t *len, libcrun_error_t *err);
 
 int read_all_file_at (int dirfd, const char *path, char **out, size_t *len, libcrun_error_t *err);
@@ -372,10 +379,10 @@ int get_file_type_fd (int fd, mode_t *mode);
 
 char *get_user_name (uid_t uid);
 
-int safe_openat (int dirfd, const char *rootfs, size_t rootfs_len, const char *path, int flags, int mode,
+int safe_openat (int dirfd, const char *rootfs, const char *path, int flags, int mode,
                  libcrun_error_t *err);
 
-ssize_t safe_write (int fd, const void *buf, ssize_t count);
+int safe_write (int fd, const char *fname, const void *buf, size_t count, libcrun_error_t *err);
 
 int append_paths (char **out, libcrun_error_t *err, ...) __attribute__ ((sentinel));
 
